// Copyright (c) 2013-2016 mogemimi. Distributed under the MIT license.

#pragma once

#include "Pomdog/Content/AssetManager.hpp"
#include "Pomdog/Graphics/detail/ForwardDeclarations.hpp"
#include "Pomdog/Math/detail/ForwardDeclarations.hpp"
#include <cstdint>
#include <memory>

namespace Pomdog {

class SpriteBatchRenderer {
public:
    SpriteBatchRenderer(
        std::shared_ptr<GraphicsDevice> const& graphicsDevice,
        AssetManager & assets);

    ~SpriteBatchRenderer();

    void Begin(
        std::shared_ptr<GraphicsCommandList> const& commandList,
        Matrix4x4 const& transformMatrix);

    void Draw(
        std::shared_ptr<Texture2D> const& texture,
        Matrix3x2 const& worldMatrix,
        Color const& color,
        Vector2 const& originPivot);

    void Draw(
        std::shared_ptr<Texture2D> const& texture,
        Matrix3x2 const& worldMatrix,
        Rectangle const& sourceRect,
        Color const& color,
        Vector2 const& originPivot);

    void Draw(
        std::shared_ptr<Texture2D> const& texture,
        Rectangle const& sourceRect,
        Color const& color);

    void Draw(
        std::shared_ptr<Texture2D> const& texture,
        Vector2 const& position,
        Color const& color);

    void Draw(
        std::shared_ptr<Texture2D> const& texture,
        Vector2 const& position,
        Rectangle const& sourceRect,
        Color const& color);

    void Draw(
        std::shared_ptr<Texture2D> const& texture,
        Vector2 const& position,
        Rectangle const& sourceRect,
        Color const& color,
        Radian<float> const& rotation,
        Vector2 const& originPivot,
        float scale);

    void Draw(
        std::shared_ptr<Texture2D> const& texture,
        Vector2 const& position,
        Rectangle const& sourceRect,
        Color const& color,
        Radian<float> const& rotation,
        Vector2 const& originPivot,
        Vector2 const& scale);

    void End();

    int GetDrawCallCount() const noexcept;

private:
    class Impl;
    std::unique_ptr<Impl> impl;
};

} // namespace Pomdog
