// Copyright (c) 2013-2016 mogemimi. Distributed under the MIT license.

#pragma once

#include "Application/Duration.hpp"
#include "Application/Game.hpp"
#include "Application/GameClock.hpp"
#include "Application/GameHost.hpp"
#include "Application/GameWindow.hpp"
#include "Application/MouseCursor.hpp"
#include "Application/TimePoint.hpp"
#include "Application/Timer.hpp"

#include "Audio/AudioChannels.hpp"
#include "Audio/AudioClip.hpp"
#include "Audio/AudioEmitter.hpp"
#include "Audio/AudioEngine.hpp"
#include "Audio/AudioListener.hpp"
#include "Audio/SoundEffect.hpp"
#include "Audio/SoundState.hpp"

#include "Content/AssetManager.hpp"
#include "Content/AssetBuilders/PipelineStateBuilder.hpp"
#include "Content/AssetBuilders/ShaderBuilder.hpp"

#include "Math/BoundingBox.hpp"
#include "Math/BoundingBox2D.hpp"
#include "Math/BoundingCircle.hpp"
#include "Math/BoundingSphere.hpp"
#include "Math/Color.hpp"
#include "Math/ContainmentType.hpp"
#include "Math/Degree.hpp"
#include "Math/MathHelper.hpp"
#include "Math/Matrix2x2.hpp"
#include "Math/Matrix3x2.hpp"
#include "Math/Matrix3x3.hpp"
#include "Math/Matrix4x4.hpp"
#include "Math/Point2D.hpp"
#include "Math/Point3D.hpp"
#include "Math/Quaternion.hpp"
#include "Math/Radian.hpp"
#include "Math/Ray.hpp"
#include "Math/Rectangle.hpp"
#include "Math/Vector2.hpp"
#include "Math/Vector3.hpp"
#include "Math/Vector4.hpp"

#include "Logging/Log.hpp"
#include "Logging/LogChannel.hpp"
#include "Logging/LogEntry.hpp"
#include "Logging/LogLevel.hpp"

#include "Graphics/Blend.hpp"
#include "Graphics/BlendDescription.hpp"
#include "Graphics/BlendOperation.hpp"
#include "Graphics/BufferUsage.hpp"
#include "Graphics/ClearOptions.hpp"
#include "Graphics/ComparisonFunction.hpp"
#include "Graphics/ConstantBuffer.hpp"
#include "Graphics/CullMode.hpp"
#include "Graphics/DepthStencilDescription.hpp"
#include "Graphics/DepthStencilOperation.hpp"
#include "Graphics/EffectAnnotation.hpp"
#include "Graphics/EffectConstantDescription.hpp"
#include "Graphics/EffectReflection.hpp"
#include "Graphics/EffectVariableClass.hpp"
#include "Graphics/EffectVariableType.hpp"
#include "Graphics/EffectVariable.hpp"
#include "Graphics/FillMode.hpp"
#include "Graphics/GraphicsCommandList.hpp"
#include "Graphics/GraphicsCommandQueue.hpp"
#include "Graphics/GraphicsDevice.hpp"
#include "Graphics/IndexBuffer.hpp"
#include "Graphics/IndexElementSize.hpp"
#include "Graphics/InputClassification.hpp"
#include "Graphics/InputElement.hpp"
#include "Graphics/InputElementFormat.hpp"
#include "Graphics/InputLayoutDescription.hpp"
#include "Graphics/InputLayoutHelper.hpp"
#include "Graphics/PipelineState.hpp"
#include "Graphics/PipelineStateDescription.hpp"
#include "Graphics/PresentationParameters.hpp"
#include "Graphics/PrimitiveTopology.hpp"
#include "Graphics/RasterizerDescription.hpp"
#include "Graphics/RenderTarget2D.hpp"
#include "Graphics/SamplerDescription.hpp"
#include "Graphics/SamplerState.hpp"
#include "Graphics/Shader.hpp"
#include "Graphics/ShaderLanguage.hpp"
#include "Graphics/ShaderPipelineStage.hpp"
#include "Graphics/StencilOperation.hpp"
#include "Graphics/SurfaceFormat.hpp"
#include "Graphics/Texture.hpp"
#include "Graphics/Texture2D.hpp"
#include "Graphics/TextureAddressMode.hpp"
#include "Graphics/TextureFilter.hpp"
#include "Graphics/VertexBuffer.hpp"
#include "Graphics/VertexBufferBinding.hpp"
#include "Graphics/Viewport.hpp"

#include "Input/ButtonState.hpp"
#include "Input/Gamepad.hpp"
#include "Input/GamepadButtons.hpp"
#include "Input/GamepadCapabilities.hpp"
#include "Input/GamepadDPad.hpp"
#include "Input/GamepadState.hpp"
#include "Input/GamepadThumbSticks.hpp"
#include "Input/GamepadType.hpp"
#include "Input/Keyboard.hpp"
#include "Input/KeyboardState.hpp"
#include "Input/Keys.hpp"
#include "Input/KeyState.hpp"
#include "Input/Mouse.hpp"
#include "Input/MouseState.hpp"
#include "Input/PlayerIndex.hpp"
#include "Input/TouchLocation.hpp"
#include "Input/TouchLocationState.hpp"

#include "Signals/Connection.hpp"
#include "Signals/ConnectionList.hpp"
#include "Signals/Event.hpp"
#include "Signals/EventQueue.hpp"
#include "Signals/Helpers.hpp"
#include "Signals/ScopedConnection.hpp"
#include "Signals/Signal.hpp"

#include "Utility/Any.hpp"
#include "Utility/Assert.hpp"
#include "Utility/Exception.hpp"
#include "Utility/StringHelper.hpp"

#include "Basic/Export.hpp"
#include "Basic/Platform.hpp"
#include "Basic/Version.hpp"
